#ifndef LCD_H
#define LCD_H

/**
 * @brief set data to LCD
 * @param[in] data data to LCD
 */
void lcd_set_data(unsigned char data);

/**
 * @brief set enable signal to LCD
 * @param[in] state state for enable signal (0 or 1)
 */
void lcd_set_e(unsigned char state);

/**
 * @brief set register select signal to LCD
 * @param[in] state state for register select signal (0 or 1)
 */
void lcd_set_rs(unsigned char state);

/**
 * @brief set read/write signal to LCD
 * @param[in] state state for read/write signal (0 or 1)
 */
void lcd_set_rw(unsigned char state);

/** set LCD to 4 bit mode */
void lcd_set4bit(void);

/**
 * @brief output a byte to LCD
 * @param[in] data if the byte is a data byte (command otherwise)
 * @param[in] byte byte to write
 */
void lcd_byte(unsigned char data, unsigned char byte);

/** initialize LCD */
void lcd_init(void);

/**
 * @brief show character
 * @param[in] line number of line (0..1)
 * @param[in] pos position in line (0..15)
 * @param[in] chr character to set
 */
void lcd_chr(unsigned int line, unsigned int pos, char chr);

/**
 * @brief show string
 * @param[in] line number of line (0..1)
 * @param[in] str string to show
 */
void lcd_str(unsigned int line, const char *str);

#endif /* #ifndef LCD_H */

