#ifndef UART_H
#define UART_H

/**
 * @brief configure baudrate scaler
 * @param[in] scale baudrate scaler (1 = 7.143 Mhz)
 */
void uart_cfg_scale(unsigned short scale);

/**
 * @brief configure number of data bits
 * @param[in] bits number of data bits
 */
void uart_cfg_bits(unsigned char bits);

/**
 * @brief configure number of stop bits
 * @param[in] stop number of stop bits
 */
void uart_cfg_stop(unsigned char stop);

/**
 * @brief check if transmitting a character is possible
 * @return if transmitting a character is possible
 */
int uart_can_tx(void);

/**
 * @brief transmit a character
 * @param[in] chr character to transmit
 */
void uart_tx(unsigned short chr);

/**
 * @brief check if receiving a character is possible
 * @return if receiving a character is possible
 */
int uart_can_rx(void);

/**
 * @brief receive a character
 * @return character received
 */
unsigned short uart_rx(void);

/**
 * @brief determine if character is an error character
 * @param[in] chr character to check
 * @return if error
 */
int uart_is_err(unsigned short chr);

#endif /* #ifndef UART_H */

